"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAwsCredentialsFromLocal = exports.getAwsCredentialsFromAuthService = exports.createPlutoRole = exports.savePlutoGlobalConfig = exports.readPlutoGlobalConfig = exports.genPulumiConfigForAWS = exports.genPulumiConfigByRuntime = void 0;
const fs_1 = __importDefault(require("fs"));
const os_1 = __importDefault(require("os"));
const path_1 = __importDefault(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const yaml = __importStar(require("js-yaml"));
const open_1 = __importDefault(require("open"));
const crypto_1 = require("crypto");
const credential_providers_1 = require("@aws-sdk/credential-providers");
const shared_ini_file_loader_1 = require("@smithy/shared-ini-file-loader");
const base_1 = require("@plutolang/base");
const PLUTO_GLOBAL_CONFIG_PATH = path_1.default.join(os_1.default.homedir(), ".pluto", "config.yml");
const AWS_CREDENTIALS_QUERY_URL = "https://ryo1adp0pe.execute-api.us-east-1.amazonaws.com/dev/query";
async function genPulumiConfigByRuntime(sta) {
    const genFnMapping = {
        [base_1.PlatformType.AWS]: genPulumiConfigForAWS,
        [base_1.PlatformType.K8s]: genPulumiConfigForK8s,
        [base_1.PlatformType.AliCloud]: genPulumiConfigForAliCloud,
    };
    if (!(sta.platformType in genFnMapping)) {
        throw new Error("Not support this runtime.");
    }
    return await genFnMapping[sta.platformType](sta);
}
exports.genPulumiConfigByRuntime = genPulumiConfigByRuntime;
async function genPulumiConfigForAWS() {
    const region = await getRegion();
    if (region == undefined) {
        throw new Error("Please make sure to set your default region by setting the AWS_REGION environment variable first.");
    }
    let creds = await getAwsCredentialsFromLocal();
    if (creds == undefined) {
        const plutoConfig = readPlutoGlobalConfig();
        if (!("userId" in plutoConfig)) {
            console.info("There is not any AWS credentials on your machine. We will guide you through the process of creating an AWS Role. We will open a new tab in your browser to create a CloudFormation stack. Please complete the creation process there.");
            const userId = await createPlutoRole();
            plutoConfig["userId"] = userId;
            savePlutoGlobalConfig(plutoConfig);
        }
        creds = await getAwsCredentialsFromAuthService(plutoConfig["userId"]);
        if (process.env.DEBUG) {
            console.debug("Got credentials from auth service.");
        }
    }
    process.env["AWS_ACCESS_KEY_ID"] = creds.accessKeyId;
    process.env["AWS_SECRET_ACCESS_KEY"] = creds.secretAccessKey;
    if (creds.sessionToken) {
        process.env["AWS_SESSION_TOKEN"] = creds.sessionToken;
    }
    return {
        "aws:region": { value: region },
    };
}
exports.genPulumiConfigForAWS = genPulumiConfigForAWS;
async function genPulumiConfigForK8s(sta) {
    const configMap = {};
    if (sta.configs["kubernetes:kubeconfig"]) {
        configMap["kubernetes:kubeconfig"] = { value: sta.configs["kubernetes:kubeconfig"] };
    }
    return configMap;
}
async function genPulumiConfigForAliCloud(sta) {
    sta;
    if (!process.env.ALICLOUD_REGION ||
        !process.env.ALICLOUD_ACCESS_KEY ||
        !process.env.ALICLOUD_SECRET_KEY) {
        throw new Error("You need to set the ALICLOUD_REGION, ALICLOUD_ACCESS_KEY, and ALICLOUD_SECRET_KEY environment variables.");
    }
    return {};
}
function readPlutoGlobalConfig() {
    let plutoConfig = {};
    if (fs_1.default.existsSync(PLUTO_GLOBAL_CONFIG_PATH)) {
        const plutoConfigText = fs_1.default.readFileSync(PLUTO_GLOBAL_CONFIG_PATH, "utf-8");
        plutoConfig = yaml.load(plutoConfigText);
    }
    return plutoConfig;
}
exports.readPlutoGlobalConfig = readPlutoGlobalConfig;
function savePlutoGlobalConfig(plutoConfig) {
    fs_extra_1.default.ensureFileSync(PLUTO_GLOBAL_CONFIG_PATH);
    fs_extra_1.default.writeFileSync(PLUTO_GLOBAL_CONFIG_PATH, yaml.dump(plutoConfig));
}
exports.savePlutoGlobalConfig = savePlutoGlobalConfig;
// TODO: move to cli, and add a role name so that users can change thier accounts.
async function createPlutoRole() {
    // Guide users to create PLRole. Open one tab in the browser.
    const userId = (0, crypto_1.randomUUID)();
    const stackId = userId.split("-")[0];
    const url = `https://us-east-1.console.aws.amazon.com/cloudformation/home?region=us-east-1#/stacks/quickcreate?templateURL=https://plutolang.s3.amazonaws.com/roleTemplate.yml&stackName=PLSetup-${stackId}&param_RoleName=PLRole-${stackId}&param_UserId=${userId}`;
    await (0, open_1.default)(url);
    let succ = false;
    const interval = 10;
    for (let i = 0; i < (3 * 60) / interval; i++) {
        await sleep(interval);
        try {
            await getAwsCredentialsFromAuthService(userId);
            succ = true;
            break;
        }
        catch (e) {
            console.log("Waiting for PLRole creation...");
        }
    }
    if (succ) {
        console.log("Successfully created PLRole!");
        return userId;
    }
    else {
        throw new Error("Failed to create PLRole.");
    }
}
exports.createPlutoRole = createPlutoRole;
async function getAwsCredentialsFromAuthService(userId) {
    const queryUrl = AWS_CREDENTIALS_QUERY_URL + `?userId=${userId}`;
    const response = await fetch(queryUrl);
    if (response.status != 200) {
        throw new Error(`Get credentials failed, ${await response.text()}.`);
    }
    const data = await response.json();
    return {
        accessKeyId: data["AccessKeyId"],
        secretAccessKey: data["SecretAccessKey"],
        sessionToken: data["SessionToken"],
    };
}
exports.getAwsCredentialsFromAuthService = getAwsCredentialsFromAuthService;
async function getAwsCredentialsFromLocal() {
    // Get credentials from envrionment variables.
    const envProvider = (0, credential_providers_1.fromEnv)();
    let identity = await envProvider().catch(() => undefined);
    if (identity != undefined) {
        return {
            accessKeyId: identity.accessKeyId,
            secretAccessKey: identity.secretAccessKey,
            sessionToken: identity.sessionToken,
        };
    }
    // Get credentials from shared INI credentials file.
    // TODO: Currently, the default path to ~/.aws/credentials is being used.
    //   Enable users to specify the path to the shared credentials file.
    const profile = process.env.AWS_PROFILE ?? "default";
    const credsFilepath = `${os_1.default.homedir()}/.aws/credentials`;
    const configFilepath = `${os_1.default.homedir()}/.aws/config`;
    const iniProvider = (0, credential_providers_1.fromIni)({
        profile: profile,
        filepath: credsFilepath,
        configFilepath: configFilepath,
    });
    identity = await iniProvider().catch(() => undefined);
    if (identity != undefined) {
        return {
            accessKeyId: identity.accessKeyId,
            secretAccessKey: identity.secretAccessKey,
            sessionToken: identity.sessionToken,
        };
    }
    return undefined;
}
exports.getAwsCredentialsFromLocal = getAwsCredentialsFromLocal;
async function getRegion() {
    if (process.env.AWS_REGION) {
        return process.env.AWS_REGION;
    }
    const profile = process.env.AWS_PROFILE ?? "default";
    const awsConfig = await (0, shared_ini_file_loader_1.loadSharedConfigFiles)();
    const profileConfig = awsConfig.configFile[profile];
    if (profileConfig) {
        const region = profileConfig["region"];
        return region;
    }
    return;
}
function sleep(sec) {
    return new Promise((resolve) => setTimeout(resolve, sec * 1000));
}
//# sourceMappingURL=utils.js.map