"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const dotenv_1 = __importDefault(require("dotenv"));
dotenv_1.default.config({ path: `.env.${process.env.NODE_ENV}.local` });
const os_1 = __importDefault(require("os"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const vitest_1 = require("vitest");
const utils_1 = require("./utils");
const crypto_1 = require("crypto");
const inLocal = process.env.LOCAL === "true";
const credsFilepath = `${os_1.default.homedir()}/.aws/credentials`;
const configFilepath = `${os_1.default.homedir()}/.aws/config`;
let oldCredentials, oldConfig, oldPlutoConfig;
// load the original aws credentials and config files before testing begins
(0, vitest_1.beforeAll)(() => {
    fs_extra_1.default.ensureFileSync(credsFilepath);
    fs_extra_1.default.ensureFileSync(configFilepath);
    oldCredentials = fs_extra_1.default.readFileSync(credsFilepath);
    oldConfig = fs_extra_1.default.readFileSync(configFilepath);
    oldPlutoConfig = (0, utils_1.readPlutoGlobalConfig)();
});
// restore the original aws credentials and config files after testing ends
(0, vitest_1.afterAll)(() => {
    fs_extra_1.default.writeFileSync(credsFilepath, oldCredentials);
    fs_extra_1.default.writeFileSync(configFilepath, oldConfig);
    (0, utils_1.savePlutoGlobalConfig)(oldPlutoConfig);
});
vitest_1.test.todo("create a Pluto Role", async () => {
    const oldPlutoConfig = (0, utils_1.readPlutoGlobalConfig)();
    (0, utils_1.savePlutoGlobalConfig)({});
    const userId = await (0, utils_1.createPlutoRole)().catch((e) => {
        console.error(e);
        return undefined;
    });
    (0, vitest_1.expect)(userId).toBeDefined();
    (0, utils_1.savePlutoGlobalConfig)(oldPlutoConfig);
});
(0, vitest_1.test)("get credentials from local", async () => {
    fs_extra_1.default.writeFileSync(credsFilepath, "");
    fs_extra_1.default.writeFileSync(configFilepath, "");
    delete process.env.AWS_ACCESS_KEY_ID;
    delete process.env.AWS_SECRET_ACCESS_KEY;
    delete process.env.REGION;
    delete process.env.AWS_PROFILE;
    // Test: file is not empty
    const strs = [(0, crypto_1.randomUUID)(), (0, crypto_1.randomUUID)(), (0, crypto_1.randomUUID)(), (0, crypto_1.randomUUID)(), (0, crypto_1.randomUUID)(), (0, crypto_1.randomUUID)()];
    const testConfig = `[default]
region = us-east-1
[profile serverless]
region = us-east-1
`;
    const testCreds = `[default]
aws_access_key_id = ${strs[0]}
aws_secret_access_key = ${strs[1]}
[serverless]
aws_access_key_id = ${strs[2]}
aws_secret_access_key = ${strs[3]}
`;
    fs_extra_1.default.writeFileSync(credsFilepath, testCreds);
    fs_extra_1.default.writeFileSync(configFilepath, testConfig);
    let creds = await (0, utils_1.getAwsCredentialsFromLocal)();
    (0, vitest_1.expect)(creds).toBeDefined();
    (0, vitest_1.expect)(creds?.accessKeyId).toEqual(strs[0]);
    (0, vitest_1.expect)(creds?.secretAccessKey).toEqual(strs[1]);
    process.env.AWS_PROFILE = "serverless";
    creds = await (0, utils_1.getAwsCredentialsFromLocal)();
    (0, vitest_1.expect)(creds).toBeDefined();
    (0, vitest_1.expect)(creds?.accessKeyId).toEqual(strs[2]);
    (0, vitest_1.expect)(creds?.secretAccessKey).toEqual(strs[3]);
    process.env.AWS_ACCESS_KEY_ID = strs[4];
    process.env.AWS_SECRET_ACCESS_KEY = strs[5];
    creds = await (0, utils_1.getAwsCredentialsFromLocal)();
    (0, vitest_1.expect)(creds).toBeDefined();
    (0, vitest_1.expect)(creds?.accessKeyId).toEqual(strs[4]);
    (0, vitest_1.expect)(creds?.secretAccessKey).toEqual(strs[5]);
});
vitest_1.test.runIf(inLocal)("get credentials from auth service", async () => {
    const userId = process.env.USER_ID;
    if (userId == undefined) {
        throw new Error("USER_ID is not defined");
    }
    const creds = await (0, utils_1.getAwsCredentialsFromAuthService)(userId).catch(() => undefined);
    (0, vitest_1.expect)(creds).toBeDefined();
});
//# sourceMappingURL=utils.test.js.map